/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */

package com.funambol.domino.engine.cache;

import com.funambol.domino.admin.DominoConnectorConfig;
import com.funambol.domino.exception.DominoConfigException;
import com.funambol.domino.util.Def;

import java.io.Serializable;
import java.security.Principal;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.logging.*;

import com.funambol.framework.core.*;
import com.funambol.framework.engine.*;
import com.funambol.framework.engine.source.*;
import com.funambol.framework.security.*;
import com.funambol.framework.tools.beans.*;
import com.funambol.framework.logging.Sync4jLogger;

/**
 * This class represents a SyncSource that read and store the state of the items
 * in a support table.
 * @author Stefano Nichele
 * @version $Id: CachedSyncSource.java,v 1.2 2006/08/08 10:56:26 adeac Exp $
 */
public abstract class CachedSyncSource
        extends AbstractSyncSource
        implements MergeableSyncSource, Serializable,
        com.funambol.framework.tools.beans.LazyInitBean{


    // ------------------------------------------------------------ Constants

    protected static final String PRODUCT_NAME = "Funambol Domino Connector";

    // ------------------------------------------------------------ Private Data

    private String jndiDataSourceName =  null;

    private ItemStateDAO itemStateDao = null;

    private Logger log                = Sync4jLogger.getLogger(Def.LOGGER_NAME);

    private SyncItemInfo[] localItems    = null;
    private SyncItemInfo[] serverItems   = null;

    private SyncItemKey[] newItemKeys    = null;
    private SyncItemKey[] deletedItemKeys = null;
    private SyncItemKey[] updatedItemKeys = null;

    protected Principal principal          = null;

    protected int       syncMode;

    /**
     * Init the syncSource
     *
     * @throws BeanInitializationException
     */
    public void init() throws BeanInitializationException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("Init CachedSyncSource with sourceUri: " + sourceURI);
        }

        try {
            jndiDataSourceName  = DominoConnectorConfig.getConfigInstance().getDataSource();
        } catch (DominoConfigException e) {
            log.severe("Error Getting jndi datasource Parameters:: " + e);
            log.throwing(getClass().getName(), "check Domino Credentials", e);
        }

        itemStateDao = new ItemStateDAO(jndiDataSourceName);
        try {
            itemStateDao.init();
        } catch (Exception e) {
            throw new BeanInitializationException("Error initializing ItemStateDAO", e);
        }


    }

    /**
     *
     * @param jndiDataSourceName String
     */
    public void setJndiDataSourceName(String jndiDataSourceName) {
        this.jndiDataSourceName = jndiDataSourceName;
    }

    /**
     *
     * @return String
     */
    public String getJndiDataSourceName() {
        return jndiDataSourceName;
    }


    /**
     * BeginSync
     *
     * @param context SyncContext
     * @throws SyncSourceException
     */
    public void beginSync(SyncContext context)
    throws SyncSourceException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("CachedSyncSource beginSync [" + sourceURI + "]");
        }

        this.principal = context.getPrincipal();
        this.syncMode  = context.getSyncMode();

        long principalId = ((Sync4jPrincipal) this.principal).getId();

        try {

            if ((syncMode == AlertCode.SLOW) ||
                    (syncMode == AlertCode.REFRESH_FROM_SERVER)) {

                itemStateDao.deleteLocalItems(this.sourceURI,principalId);

            } else {

                if (syncMode == AlertCode.REFRESH_FROM_CLIENT) {


                    itemStateDao.deleteLocalItems(this.sourceURI,principalId);
                    removeAllItems();

                }

                this.serverItems = getAllSyncItemInfo();
                this.localItems  = itemStateDao.getLocalItems(this.sourceURI,
                        principalId);
                this.newItemKeys    = findNewItemIds    () ;
                this.updatedItemKeys = findUpdateItemIds() ;
                this.deletedItemKeys = findDeleteItemIds() ;

            }
        } catch (Exception e) {
            throw new SyncSourceException("Error accessing items: " +
                    e.getMessage(), e);
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("NEW Item keys");
            if (this.newItemKeys != null){
                log.finest("NEW Item keys lenght:" + this.newItemKeys.length);
            } else {
                log.finest("There is no NEW Item keys");
            }

            log.finest("UPDATED Item keys");
            if (this.updatedItemKeys != null){
                log.finest("UPDATED Item keys lenght:" + this.updatedItemKeys.length);
            } else {
                log.finest("There is no UPDATED Item keys");
            }

            log.finest("DELETED Item keys");
            if (this.deletedItemKeys != null){
                log.finest("DELETED Item keys lenght:" + this.deletedItemKeys.length);
            } else {
                log.finest("There is no DELETED Item keys");
            }
        }


        if (log.isLoggable(Level.FINEST)) {
            log.finest("CachedSyncSource beginSync done [" + sourceURI + "]");
        }

    }

    /**
     * EndSync
     *
     * @throws SyncSourceException
     */
    public void endSync() throws SyncSourceException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("CachedSyncSource endSync [" + sourceURI + "]");
        }

        SyncItemInfo[] serverItems;
        long principalId = ((Sync4jPrincipal) principal).getId();

        try {
            serverItems  = getAllSyncItemInfo();

            itemStateDao.updateLocalItems(serverItems ,
                    this.sourceURI ,
                    principalId);

        } catch (Exception e) {
            throw new SyncSourceException("Error caching items:" +
                    e.getMessage(), e);
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("CachedSyncSource endSync done [" + sourceURI + "]");
        }

    }

    /* @see com.funambol.framework.engine.source.SyncSource */
    public SyncItemKey[] getAllSyncItemKeys() throws SyncSourceException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("getAllSyncItemKeys start ");
        }

        SyncItemInfo[] itemsInfo = getAllSyncItemInfo();
        SyncItemKey[] keys       = new SyncItemKey[itemsInfo.length];

        for (int i = 0; i < itemsInfo.length; i++) {
            keys[i] = itemsInfo[i].getKey();
        }

        return keys;

    }

    /* @see com.funambol.framework.engine.source.SyncSource */
    public SyncItemKey[] getUpdatedSyncItemKeys(Timestamp since, Timestamp until)
    throws SyncSourceException {
        return updatedItemKeys;
    }

    /* @see com.funambol.framework.engine.source.SyncSource */
    public SyncItemKey[] getNewSyncItemKeys(Timestamp since, Timestamp until) throws SyncSourceException {
        return newItemKeys;
    }


    /* @see com.funambol.framework.engine.source.SyncSource */
    public SyncItemKey[] getDeletedSyncItemKeys(Timestamp since, Timestamp until)
    throws SyncSourceException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("getDeletedSyncItemKeys [" + sourceURI + "]");
        }
        return deletedItemKeys;
    }


    /* @see com.funambol.framework.engine.source.SyncSource */
    public void setOperationStatus(String operation, int statusCode, SyncItemKey[] keys) {

        if (log.isLoggable(Level.FINEST)) {

            StringBuffer message = new StringBuffer("Received status code '");
            message.append(statusCode).append("' for a '").
                    append(operation).append("'").append(" for this items: ");

            for (int i = 0; i < keys.length; i++) {
                message.append("\n- ").append(keys[i].getKeyAsString());
            }

            log.finest(message.toString());
        }
    }

    /**
     *
     * @return SyncItemInfo[]
     * @throws SyncSourceException
     */
    public abstract SyncItemInfo[] getAllSyncItemInfo() throws SyncSourceException;

    protected abstract void removeAllItems() throws SyncSourceException;

    // ---------------------------------------------------------- Private Methods

    /**
     * Creates new items id array
     *
     * @return new items id array
     */
    private SyncItemKey[] findNewItemIds() {

        ArrayList  newRows;
        SyncItemInfo serverItem;

        newRows = new ArrayList();

        for (int i = 0, l = this.serverItems.length; i < l; i++) {

            serverItem = this.serverItems[i];

            if (!isItemInArray(this.localItems, serverItem)) {
                newRows.add(serverItem.getKey());
            }

        }

        return (SyncItemKey[])newRows.toArray(new SyncItemKey[0]) ;
    }

    /**
     * Creates deleted items id array
     *
     * @return deleted items id array
     */
    private SyncItemKey[] findDeleteItemIds() {

        ArrayList  deleteRows;
        SyncItemInfo localItem;

        deleteRows = new ArrayList();

        for (int i=0, l = this.localItems.length; i < l; i++) {

            localItem = localItems[i];

            if (!this.isItemInArray(this.serverItems, localItem)) {
                deleteRows.add(localItem.getKey());
            }

        }

        return (SyncItemKey[])deleteRows.toArray(new SyncItemKey[0]);
    }

    /**
     * Creates updated items id array
     *
     * @return updated items id array
     */
    private SyncItemKey[] findUpdateItemIds() {

        ArrayList updateRows        = new ArrayList();

        SyncItemInfo serverItem;
        SyncItemInfo localItem;

        Timestamp serverItemLastModified;
        Timestamp localItemLastModified;

        for (int i = 0, l = this.serverItems.length; i < l; i++) {

            serverItem     = this.serverItems[i];
            localItem      = this.getItemById(this.localItems,
                    serverItem.getKey().getKeyAsString()) ;

            if (localItem != null) {
                serverItemLastModified      = serverItem.getLastModified() ;
                localItemLastModified    = localItem.getLastModified    () ;

                if (serverItemLastModified.getTime() != localItemLastModified.getTime()) {
                    updateRows.add(serverItem.getKey());
                }
            }
        }

        return (SyncItemKey[])updateRows.toArray(new SyncItemKey[0]) ;
    }


    /**
     * Checks if item id is included in ids from array of items
     *
     * @param items items array
     * @param id id to find
     * @return <p>true</p> if the item is included
     *         <p>false</p> if the item is not included
     **/
    private boolean isIdInArray(SyncItemInfo[] items,
                                String id) {

        boolean isFind = false;

        for (int i = 0, l = items.length; i < l; i++) {

            if (items[i].getKey().getKeyAsString().equals(id)) {
                isFind = true;
                break;
            }
        }

        return isFind;
    }

    /**
     * Checks if item is included in array of items
     *
     * @param items items array
     * @param item item to find
     * @return <p>true</p> if the item is included
     *         <p>false</p> if the item is not included
     **/
    private boolean isItemInArray(SyncItemInfo[] items,
                                  SyncItemInfo   item) {

        return isIdInArray(items, item.getKey().getKeyAsString());

    }

    /**
     * Gets item by id
     *
     * @param items array
     * @param id
     * @return <p>item</p> if found
     *         <p>null</p> if not found
     **/
    private SyncItemInfo getItemById(SyncItemInfo[] items,
                                     String id) {

        SyncItemInfo item = null;

        for (int i = 0, l = items.length; i < l; i++) {
            if (items[i].getKey().getKeyAsString().equals(id)) {
                item = items[i];
                break;
            }
        }
        return item;
    }


}