/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */

package com.funambol.domino.engine.cache;

import com.funambol.domino.util.Def;
import javax.sql.DataSource;
import javax.naming.NamingException;
import javax.naming.InitialContext;
import java.util.Date;
import java.util.logging.Logger;
import java.util.logging.Level;
import java.sql.PreparedStatement;
import java.sql.Connection;
import java.sql.Timestamp;
import com.funambol.framework.tools.DBTools;
import com.funambol.framework.logging.Sync4jLogger;

import java.sql.ResultSet;
import java.util.ArrayList;

/**
 * Contains methods to access to the table with the state of the items.
 *
 * @author Stefano Nichele
 * @version $Id: ItemStateDAO.java,v 1.1 2006/08/08 08:31:00 gibi Exp $
 */
public class ItemStateDAO {

    // --------------------------------------------------------------- Constants
    private static final String SQL_SELECT_ITEMS_BY_PRINCIPAL_BY_SOURCE    =
        "select id, last_modified " +
        "from fnbl_domino_cache where source_uri = ? and principal = ?";

    private static final String SQL_ADD_LOCAL_ITEM                         =
        "insert into fnbl_domino_cache " +
        "(id, last_modified, source_uri, principal) values(?, ?, ?, ?)"                                                   ;

    private static final String SQL_DELETE_LOCAL_ITEMS                     =
        "delete from fnbl_domino_cache where source_uri = ? and principal = ? ";


    private Logger log = Sync4jLogger.getLogger(Def.LOGGER_NAME);

    // ------------------------------------------------------------ Private data

    private String jndiDataSourceName;

    private DataSource dataSource;

    public String getJndiDataSourceName() {
        return jndiDataSourceName;
    }

    public void setJndiDataSourceName(String jndiDataSourceName) {
        this.jndiDataSourceName = jndiDataSourceName;
    }

    // ------------------------------------------------------------- Constructor
    private ItemStateDAO() {
    }

    public ItemStateDAO(String jndiDataSourceName) {
        this.jndiDataSourceName = jndiDataSourceName;
    }


    // ---------------------------------------------------------- Public Methods
    public void init() throws Exception {
        lookupDataSource();
    }


    /**
     * Adds local items
     *
     * @param items
     * @param sourceURI
     * @param principal
     *
     * @throws Exception
     **/
    public void addLocalItems(SyncItemInfo[] items,
                              String sourceURI,
                              long principal) throws Exception {

         Connection         connection    = null ;
         PreparedStatement  ps            = null ;

         String             id            = null;
         Timestamp          lastModified  = null;

         try {

            connection = dataSource.getConnection();

            for (int i=0, l = items.length; i < l; i++) {

                ps = connection.prepareStatement(SQL_ADD_LOCAL_ITEM);

                id            = items[i].getKey().getKeyAsString();
                lastModified  = items[i].getLastModified() ;

                ps.setString    (1, id                                    ) ;
                ps.setTimestamp (2, lastModified                          ) ;
                ps.setString    (3, sourceURI                             ) ;
                ps.setLong      (4, principal                             ) ;

                ps.executeUpdate();

                ps.close() ;
                ps = null  ;
            }
         } catch (Exception e) {
             throw new Exception("Error adding local items", e);
         } finally {
             DBTools.close(connection, ps, null);
         }
    }

    /**
     * Delete local items
     *
     * @param sourceURI
     * @param principal
     *
     * @throws Exception
     **/
    public void deleteLocalItems(String sourceURI, long principal)
        throws Exception {

        Connection         connection  = null ;
        PreparedStatement  ps          = null ;

        try {

            connection = dataSource.getConnection();

            ps = connection.prepareStatement(SQL_DELETE_LOCAL_ITEMS);

            ps.setString  (1, sourceURI ) ;
            ps.setLong    (2, principal ) ;

            ps.executeUpdate();

        } catch (Exception e) {
            throw new Exception("Error deleting local items", e);
        } finally {
            DBTools.close(connection, ps, null);
        }
    }

    /**
     * Update local items
     *
     * @param localItems
     * @param sourceURI
     * @param principal
     *
     * @throws Exception
     **/
    public void updateLocalItems(SyncItemInfo[] localItems ,
                                 String sourceURI  ,
                                 long principal  )
    throws Exception {

         this.deleteLocalItems (sourceURI, principal);
         this.addLocalItems    (localItems, sourceURI, principal);
    }

    /**
     * Gets local items
     *
     * @param sourceURI
     * @param principalId
     *
     * @return array of find local items
     *
     * @throws Exception
     **/
    public SyncItemInfo[] getLocalItems (String sourceURI, long principalId)
        throws Exception {

         Connection connection  = null;
         PreparedStatement ps   = null;
         ResultSet rs           = null;

         ArrayList rows         = null;

         SyncItemInfo[] items = null;

         try {

             connection = dataSource.getConnection();

             rows = new ArrayList();

             ps = connection.prepareStatement(
                    SQL_SELECT_ITEMS_BY_PRINCIPAL_BY_SOURCE);

             ps.setString(1, sourceURI);
             ps.setLong  (2, principalId);

             rs = ps.executeQuery();

             while(rs.next()) {
                 rows.add(new SyncItemInfo(rs.getString(1), rs.getTimestamp(2)));
             }

             items = (SyncItemInfo[]) rows.toArray(new SyncItemInfo[0]);

         } catch (Exception e) {
             throw new Exception("Error getting local items", e);
         } finally {
             DBTools.close(connection, ps, rs);
         }

         return items;
    }



    // --------------------------------------------------------- Private Methods

    /**
     * Lookups the datasource
     *
     * JBOSS
     * private String jndiDataSourceName = "java:jdbc/sync4j";
     *
     * TOMCAT
     * private String jndiDataSourceName = "java:comp/env/jdbc/sync4j";
     *
     * @throws Exception if a error occurs during lookup
     */
    private void lookupDataSource() throws Exception {

        InitialContext ctx = new InitialContext();
        if (log.isLoggable(Level.FINEST)) {
              log.finest("CachedSyncSource: datasource lookup");
        }

        try {
          // try tomcat  i.e. java:comp/env/jdbc/sync4j
          dataSource = (DataSource) ctx.lookup("java:comp/env/"+jndiDataSourceName) ;
          if (log.isLoggable(Level.FINEST)) {
             log.finest("CachedSyncSource: datasource lookup done using java:comp/env/ prefix");
          }
        }
        catch (NamingException ne) {
          if (log.isLoggable(Level.FINEST)) {
              log.finest("CachedSyncSource: No tomcat datasource lookup!");
          }
          try {
              // try jboss i.e. java:jdbc/sync4j
              dataSource = (DataSource) ctx.lookup("java:"+jndiDataSourceName) ;
              if (log.isLoggable(Level.FINEST)) {
                 log.finest("CachedSyncSource: datasource lookup done using java: prefix");
              }
          }
          catch (Exception ex){
            if (log.isLoggable(Level.FINEST)) {
                log.finest("CachedSyncSource: No jboss datasource lookup!");
            }
            try {
                // try othser j2ee server jdbc/sync4j
                dataSource = (DataSource) ctx.lookup(""+jndiDataSourceName) ;
                if (log.isLoggable(Level.FINEST)) {
                   log.finest("CachedSyncSource: datasource lookup done using - prefix");
                }
            }
            catch (Exception e){
              if (log.isLoggable(Level.FINEST)) {
                log.finest("CachedSyncSource: No j2ee server datasource lookup!");
              }
              throw new Exception("Data source "+ jndiDataSourceName
                                  + " not found " + e.toString());
            }
          }
        }
    }

}