/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.domino.engine.source;

import com.funambol.domino.admin.DominoConnectorConfig;
import com.funambol.domino.engine.cache.CachedSyncSource;
import com.funambol.domino.exception.DominoAccessException;
import com.funambol.domino.exception.DominoConfigException;
import com.funambol.domino.util.Def;
import com.funambol.domino.util.DominoWrapper;
import com.funambol.domino.util.Utility;
import com.funambol.framework.engine.source.MergeableSyncSource;
import com.funambol.framework.engine.source.SyncContext;
import com.funambol.framework.engine.source.SyncSource;
import com.funambol.framework.engine.source.SyncSourceException;
import com.funambol.framework.logging.Sync4jLogger;
import com.funambol.framework.security.Sync4jPrincipal;
import com.funambol.framework.server.Sync4jDevice;
import com.funambol.framework.server.store.PersistentStore;
import com.funambol.framework.server.store.PersistentStoreException;
import com.funambol.server.config.Configuration;

import java.io.Serializable;
import java.util.TimeZone;
import java.util.logging.Level;
import java.util.logging.Logger;


/**
 * This class implements common methods of
 * <pre>
 *  DominoCalendarSyncSource
 *  DominoContactSyncSource
 * </pre>
 *
 * @author Gilberto Migliavacca
 *         version $Id: DominoSyncSource.java,v 1.3 2006/09/07 15:02:34 diogo Exp $
 */
public abstract class DominoSyncSource extends CachedSyncSource implements
        MergeableSyncSource, Serializable {

    // --------------------------------------------------------------- Constants
    protected Logger log = Sync4jLogger.getLogger(Def.LOGGER_NAME);

    protected static final String TYPE_VCARD = "text/x-vcard";
    protected static final String TYPE_ICAL = "text/x-vcalendar";

    // -------------------------------------------------------------- Properties

    /**
     * Property for sync session
     */
    protected Sync4jDevice device = null;

    /**
     * Property for sync session
     */
    protected String deviceTimeZoneDescr = null;

    /**
     * Property for sync session
     */
    protected TimeZone deviceTimeZone = null;

    /**
     * Property for sync session
     */
    protected String deviceCharset = null;


    /**
     * Property domino view name
     */
    protected String viewname = "";


    /**
     * Getter for property domino view name.
     *
     * @return Value of property domino view name.
     */
    public String getViewName() {
        return viewname;
    }

    /**
     * Setter for property domino view name.
     *
     * @param viewname_e String. New value of property domino view name.
     */
    public void setViewName(String viewname_e) {
        this.viewname = viewname_e;
    }

    /**
     * Property domino Server Host
     */
    private String host = "";

    /**
     * Getter for property domino Server Host.
     *
     * @return Value of property domino Server Host.
     */
    public String getHost() {
        return host;
    }

    /**
     * Setter for property domino Server Host.
     *
     * @param hostEx String New value of property domino Server Host.
     */
    public void setHost(String hostEx) {
        this.host = hostEx;
    }

    /**
     * Property Domino Server port
     */
    private int port = 63148;

    /**
     * Getter for property Domino Server port.
     *
     * @return Value of property Domino Server port.
     */
    public int getPort() {
        return port;
    }

    /**
     * Setter for property Domino Server port.
     *
     * @param portEx int. New value of property Domino Server port.
     */
    public void setPort(int portEx) {
        this.port = portEx;
    }

    /**
     * item format
     */

    /**
     * Getter for property itemFormat.
     *
     * @return Value of property itemFormat.
     */
    public String getItemFormat() {
        return this.getType();
    }


    /**
     *
     */
    private DominoWrapper dominoWrapper = null;

    /**
     * Getter for property itemFormat.
     *
     * @return Value of property itemFormat.
     */
    public DominoWrapper getDominoWrapper() {
        return dominoWrapper;
    }

    /**
     * Property conflict resolution value
     */
    protected int conflictResolution = -1;

    /**
     * Getter for property conflictResolution.
     *
     * @return Value of property conflictResolution.
     */
    public int getConflictResolution() {
        return conflictResolution;
    }

    // ---------------------------------------------------------- Public Methods

    /**
     * @param context SyncContext
     * @throws SyncSourceException
     * @see SyncSource
     */
    public void beginSync(SyncContext context) throws SyncSourceException {

        if (log.isLoggable(Level.FINEST)) {
            log.finest("DominoSyncSource - Start beginSync [" + sourceURI + "]");
        }

        // set common properties
        this.principal = context.getPrincipal();
        this.syncMode = context.getSyncMode();
        this.conflictResolution = context.getConflictResolution();

        long principalId;
        String username;
        String credentials;
        String password;

        principalId = ((Sync4jPrincipal) this.principal).getId();
        username = ((Sync4jPrincipal) this.principal).getUsername();
        credentials = ((Sync4jPrincipal) this.principal).getEncodedCredentials();
        password = Utility.getPassword(username, credentials);

        if (log.isLoggable(Level.FINEST)) {
            log.finest("\n COORDS " +
                    "\n principalId " + principalId +
                    "\n host   " + host +
                    "\n port   " + port +
                    "\n login  " + username +
                    "\n format " + this.getType() +
                    "\n View   " + this.getViewName());
        }

        // open Domino connection
        dominoWrapper = new DominoWrapper();
        try {
            boolean ssl = DominoConnectorConfig.getConfigInstance().getSsl();
            dominoWrapper.setSession(host, port, username, password, ssl);
            dominoWrapper.setProperties(host);
        } catch (DominoAccessException e) {
            throw new SyncSourceException("Error Connecting Domino:" +
                    e.getMessage(), e);
        } catch (DominoConfigException e) {
            throw new SyncSourceException("Error Getting Domino Server Configurations:" +
                    e.getMessage(), e);
        }

        String deviceId = null;
        try {
            deviceId = ((Sync4jPrincipal) principal).getDeviceId();
            device = getDevice(deviceId);

            deviceCharset = device.getCharset();

            String timezone = context.getPrincipal().getDevice().getTimeZone();
            if (context.getPrincipal().getDevice().getConvertDate()) {
                if (timezone != null && timezone.length() > 0) {
                    deviceTimeZoneDescr = timezone;
                    deviceTimeZone = TimeZone.getTimeZone(deviceTimeZoneDescr);
                }
            }

        } catch (PersistentStoreException ex) {
            throw new SyncSourceException("Error reading device '" + deviceId + "'", ex);
        }

        super.beginSync(context);
        
        if (log.isLoggable(Level.FINEST)) {
            log.finest("DominoSyncSource - End beginSync [" + sourceURI + "]");
        }

    }

    /**
     * @throws SyncSourceException
     * @see SyncSource
     */
    public void endSync() throws SyncSourceException {
        super.endSync();

        // close Domino connection
        try {
            dominoWrapper.releaseProperties();
            dominoWrapper.releaseSession();
        } catch (DominoAccessException e) {
            throw new SyncSourceException("Error Closing Domino:" +
                    e.getMessage(), e);
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("Domino SyncSource - end EndSync [" + sourceURI + "]");
        }
    }

    // --------------------------------------------------------- Private Methods

    /**
     * Return the device with the given deviceId
     *
     * @param deviceId String
     * @return Sync4jDevice
     * @throws PersistentStoreException
     */
    private Sync4jDevice getDevice(String deviceId) throws PersistentStoreException {
        Sync4jDevice device = new Sync4jDevice(deviceId);
        PersistentStore store = Configuration.getConfiguration().getStore();
        store.read(device);
        return device;
    }

    //---------------------------------------------------------- PROTECTED METHODS

}