/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */

package com.funambol.domino.items.model;

import com.funambol.domino.exception.TaskParseException;
import com.funambol.domino.util.DateTools;
import com.funambol.domino.xml.XmlParseException;
import com.funambol.domino.xml.XmlParser;
import com.funambol.foundation.common.merge.MergeResult;
import com.funambol.foundation.pdi.utils.PDIMergeUtils;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.TimeZone;

/**
 * Represent an Exchange Task
 *
 * @version $Id: ItemTask.java,v 1.1 2006/08/08 08:33:33 gibi Exp $
 **/
public class ItemTask {

    //---------------------------------------------------------------- Constants

    // Define a common initial size for the StringBuffer.
    protected static final int  INITIAL_SIZE_SBUFFER = 2048; // 2K

    private static final String PATTERN_DAY_FORMAT = "yyyy-MM-dd";
    private static final String PATTERN_ISO8601_FORMAT = "yyyyMMdd'T'HHmmss'Z'";

    private static final String TAG_PP_ACTUAL_WORK         = "ActualWork"        ;
    private static final String TAG_PP_BILLING_INFORMATION = "BillingInformation";
    private static final String TAG_PP_CATEGORIES          = "Categories"        ;
    private static final String TAG_PP_COMPANIES           = "Companies"         ;
    private static final String TAG_PP_COMPLETE            = "Complete"          ;
    private static final String TAG_PP_DATE_COMPLETED      = "DateCompleted"     ;
    private static final String TAG_PP_DUE_DATE            = "DueDate"           ;
    private static final String TAG_PP_IMPORTANCE          = "Importance"        ;
    private static final String TAG_PP_MILEAGE             = "Mileage"           ;
    private static final String TAG_PP_NOTE                = "Body"              ;
    private static final String TAG_PP_OWNER               = "Owner"             ;
    private static final String TAG_PP_PERCENT_COMPLETE    = "PercentComplete"   ;
    private static final String TAG_PP_REMINDER_SET        = "ReminderSet"       ;
    private static final String TAG_PP_REMINDER_TIME       = "ReminderTime"      ;
    private static final String TAG_PP_SENSITIVITY         = "Sensitivity"       ;
    private static final String TAG_PP_DATE                = "StartDate"         ;
    private static final String TAG_PP_STATUS              = "Status"            ;
    private static final String TAG_PP_SUBJECT             = "Subject"           ;
    private static final String TAG_PP_TEAM_TASK           = "TeamTask"          ;
    private static final String TAG_PP_TOTAL_WORK          = "TotalWork"         ;

    //------------------------------------------------------------- Private data
    private int     actualWork         = 0     ;
    private String  billingInformation = null  ;
    private String  categories         = null  ;
    private String  companies          = null  ;
    private boolean complete           = false ;
    private Date    date               = null  ;
    private Date    dateCompleted      = null  ;
    private Date    dueDate            = null  ;
    private String  importance         = null  ;
    private String  mileage            = null  ;
    private String  owner              = null  ;
    private String  percentComplete    = null  ;
    private boolean reminderSet        = false ;
    private Date    reminderTime       = null  ;
    private String  sensitivity        = null  ;
    private String  status             = null  ;
    private String  subject            = null  ;
    private boolean teamTask           = false ;
    private String  textDescription    = null  ;
    private int     totalWork          = 0     ;

    private String  href               = null  ;

    private String  username           = null  ;
    private String  id                 = null  ;

    private Date    lastModified       = null  ;
    private char    state              = ' '   ;
    private TimeZone timeZone = null;

    //------------------------------------------------------------- Constructors

    /**
     * Creates a Task
     *
     */
    public ItemTask() {
    }

    /**
     * Creates a Task
     *
     * @param id client item id
     */
    public ItemTask(String id) {
        this.id = id;
    }

    /**
     * Creates a Task
     *
     * @param id client item id
     * @param t client item timestamp
     */
    public ItemTask(String id, Date t){
        this.id           = id;
        this.lastModified = t ;
    }

    /**
     * Creates a Task
     *
     * @param id client item id
     * @param state client item state
     */
    public ItemTask(String id, char state) {
        this.id    = id   ;
        this.state = state;
    }

    /**
     * Creates a Task
     *
     * @param id
     * @param href
     * @param state
     * @param lastModified
     */
    public ItemTask(String id          ,
            String href        ,
            char   state       ,
            Date   lastModified) {

        this.id           = id          ;
        this.href         = href        ;
        this.state        = state       ;
        this.lastModified = lastModified;
    }

    /**
     * Creates a Task
     *
     * @param id client item id
     * @param content client item content
     * @param t client item timestamp
     */
    public ItemTask(String id, String content, Date t, TimeZone tz) throws TaskParseException {
        getPPTask(id, content, t, tz);
    }

    //----------------------------------------------------------- Public methods
    public String getId() {
        return this.id;
    }

    public String getUsername() {
        return this.username;
    }

    public Date getDate() {
        return this.date;
    }

    public String getHref() {
        return this.href;
    }

    public Date getLastModified() {
        return this.lastModified;
    }

    public char getState() {
        return this.state;
    }

    public int getActualWork() {
        return this.actualWork;
    }

    public String getBillingInformation() {
        return this.billingInformation;
    }

    public String getCompanies() {
        return this.companies;
    }

    public String getCategories() {
        return categories;
    }

    public Date getDateCompleted() {
        return this.dateCompleted;
    }

    public Date getDueDate() {
        return dueDate;
    }

    public String getImportance() {
        return importance;
    }

    public String getMileage() {
        return mileage;
    }

    public String getOwner() {
        return this.owner;
    }

    public String getPercentComplete() {
        return percentComplete;
    }

    public boolean getReminderSet() {
        return reminderSet;
    }

    public Date getReminderTime() {
        return reminderTime;
    }

    public String getSensitivity() {
        return sensitivity;
    }

    public String getStatus() {
        return status;
    }

    public String getSubject() {
        return this.subject;
    }

    public boolean getTeamTask() {
        return this.teamTask;
    }

    public boolean isTeamTask() {
        return this.teamTask;
    }

    public String getTextDescription() {
        return this.textDescription;
    }

    public int getTotalWork() {
        return this.totalWork;
    }

    public boolean isComplete() {
        return this.complete;
    }

    public void setId(String id) {
        this.id = id;
    }

    public void setUsername(String username) {
        this.username = username;
    }

    public void setState(char state) {
        this.state = state;
    }

    public void setDate(Date date) {
        this.date = date;
    }

    public void setHref(String href) {
        this.href = href;
    }

    public void setLastModified(Date lastModified) {
        this.lastModified = lastModified;
    }

    public void setActualWork(int actualWork) {
        this.actualWork = actualWork;
    }

    public void setBillingInformation(String billingInformation) {
        this.billingInformation = billingInformation;
    }

    public void setCompanies(String companies) {
        this.companies = companies;
    }

    public void setComplete(boolean complete) {
        this.complete = complete;
    }

    public void setCategories(String categories) {
        this.categories = categories;
    }

    public void setDateCompleted(Date dateCompleted) {
        this.dateCompleted = dateCompleted;
    }

    public void setDueDate(Date dueDate) {
        this.dueDate = dueDate;
    }

    public void setImportance(String importance) {
        this.importance = importance;
    }

    public void setMileage(String mileage) {
        this.mileage = mileage ;
    }

    public void setOwner(String owner) {
        this.owner = owner;
    }

    public void setPercentComplete(String percentComplete) {
        this.percentComplete = percentComplete ;
    }

    public void setReminderSet(boolean reminderSet) {
        this.reminderSet = reminderSet ;
    }

    public void setReminderTime(Date reminderTime) {
        this.reminderTime = reminderTime;
    }

    public void setSensitivity(String sensitivity) {
        this.sensitivity = sensitivity;
    }

    public void setStatus(String status) {
        this.status = status;
    }

    public void setSubject(String subject) {
        this.subject = subject;
    }

    public void setTeamTask(boolean teamTask) {
        this.teamTask = teamTask;
    }

    public void setTextDescription(String textDescription) {
        this.textDescription = textDescription;
    }

    public void setTotalWork(int totalWork) {
        this.totalWork = totalWork;
    }
    /**
     * Set a Task properties from Client format content
     *
     * @param id client item id
     * @param content client item content
     * @param t client item timestamp
     */
    public void getPPTask(String id, String content, Date t, TimeZone tz)
    throws TaskParseException {

        String tmp;

        this.id           = id;
        this.lastModified = t ;
        this.timeZone = tz;

        try {

            this.date               = getDate(XmlParser.getXMLInitTagValue(content, TAG_PP_DATE), PATTERN_DAY_FORMAT, tz);
            tmp                     = XmlParser.getXMLInitTagValue(content, TAG_PP_ACTUAL_WORK);
            if (tmp != null && tmp.length() > 0) {
                this.actualWork = Integer.parseInt(tmp);
            }

            this.billingInformation = XmlParser.getXMLInitTagValue(content, TAG_PP_BILLING_INFORMATION);
            this.categories         = XmlParser.getXMLInitTagValue(content, TAG_PP_CATEGORIES);
            this.companies          = XmlParser.getXMLInitTagValue(content, TAG_PP_COMPANIES);
            this.dateCompleted      = getDate(XmlParser.getXMLInitTagValue(content, TAG_PP_DATE_COMPLETED ), PATTERN_DAY_FORMAT, tz);
            this.dueDate            = getDate(XmlParser.getXMLInitTagValue(content, TAG_PP_DUE_DATE), PATTERN_DAY_FORMAT, tz);
            this.importance         = XmlParser.getXMLInitTagValue(content, TAG_PP_IMPORTANCE);
            this.mileage            = XmlParser.getXMLInitTagValue(content, TAG_PP_MILEAGE);
            this.owner              = XmlParser.getXMLInitTagValue(content, TAG_PP_OWNER);
            this.percentComplete    = XmlParser.getXMLInitTagValue(content, TAG_PP_PERCENT_COMPLETE);

            this.complete           = sifValueToBoolean(XmlParser.getXMLInitTagValue(content, TAG_PP_COMPLETE));
            this.reminderSet        = sifValueToBoolean(XmlParser.getXMLInitTagValue(content, TAG_PP_REMINDER_SET));

            this.teamTask           = sifValueToBoolean(XmlParser.getXMLInitTagValue(content, TAG_PP_TEAM_TASK));


            this.subject         = XmlParser.getXMLInitTagValue(content, TAG_PP_SUBJECT);
            this.textDescription = XmlParser.getXMLInitTagValue(content, TAG_PP_NOTE);


            tmp = XmlParser.getXMLInitTagValue(content, TAG_PP_REMINDER_TIME);
            if (tmp != null && !tmp.equals("")) {
                this.reminderTime = getDate(tmp);
            }
            this.sensitivity     = XmlParser.getXMLInitTagValue(content, TAG_PP_SENSITIVITY);
            this.status          = XmlParser.getXMLInitTagValue(content, TAG_PP_STATUS);

            tmp                  = XmlParser.getXMLInitTagValue(content, TAG_PP_TOTAL_WORK);

            if (tmp != null && tmp.length() > 0) {
                this.totalWork = Integer.parseInt(tmp);
            }

        } catch (NumberFormatException e) {
            throw new TaskParseException(e.getMessage());
        } catch (XmlParseException e) {
            throw new TaskParseException(e.getMessage());
        } catch (Exception e) {
            throw new TaskParseException(e.getMessage());
        }

    }

    /**
     * Set a Task properties to Client format content
     *
     * @return content in client format
     */
    public String toXml() throws TaskParseException {

        StringBuffer xmlMsg = new StringBuffer(INITIAL_SIZE_SBUFFER);
        String date;

        try {

            xmlMsg.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?><task>\n");

            xmlMsg.append("<"                        )
            .append(TAG_PP_ACTUAL_WORK         )
            .append(">"                        )
            .append(String.valueOf(actualWork) )
            .append("</"                       )
            .append(TAG_PP_ACTUAL_WORK         )
            .append(">"                        );

            if (categories != null) {
                xmlMsg.append("<"               )
                .append(TAG_PP_CATEGORIES )
                .append(">"               )
                .append(categories        )
                .append("</"              )
                .append(TAG_PP_CATEGORIES )
                .append(">"               );
            }

            if (companies != null) {
                xmlMsg.append("<"              )
                .append(TAG_PP_COMPANIES )
                .append(">"              )
                .append(companies        )
                .append("</"             )
                .append(TAG_PP_COMPANIES )
                .append(">"              );
            }

            xmlMsg.append("<"                         )
            .append(TAG_PP_COMPLETE             )
            .append(">"                         )
            .append(booleanToSIFValue(complete) )
            .append("</"                        )
            .append(TAG_PP_COMPLETE             )
            .append(">"                         );

            date = formatDate(this.date, PATTERN_DAY_FORMAT, timeZone);

            if (date != null && date.length() > 0) {
                xmlMsg.append("<"         )
                .append(TAG_PP_DATE )
                .append(">"         )
                .append(date        )
                .append("</"        )
                .append(TAG_PP_DATE )
                .append(">"         );
            } else {
                xmlMsg.append("<"         )
                .append(TAG_PP_DATE )
                .append(">"         )
                .append("</"        )
                .append(TAG_PP_DATE )
                .append(">"         );
            }

            if (dateCompleted != null) {
                xmlMsg.append("<"                                   )
                .append(TAG_PP_DATE_COMPLETED                 )
                .append(">"                                   )
                .append(formatDate(dateCompleted, PATTERN_DAY_FORMAT, timeZone))
                .append("</"                                  )
                .append(TAG_PP_DATE_COMPLETED                 )
                .append(">"                                   );
            }

            if (dueDate != null) {
                xmlMsg.append("<"                             )
                .append(TAG_PP_DUE_DATE                 )
                .append(">"                             )
                .append(formatDate(dueDate, PATTERN_DAY_FORMAT, timeZone))
                .append("</"                            )
                .append(TAG_PP_DUE_DATE                 )
                .append(">"                             );
            } else {
                xmlMsg.append("<"             )
                .append(TAG_PP_DUE_DATE )
                .append(">"             )
                .append("</"            )
                .append(TAG_PP_DUE_DATE )
                .append(">"             );
            }

            if (importance != null) {
                xmlMsg.append("<"               )
                .append(TAG_PP_IMPORTANCE )
                .append(">"               )
                .append(importance        )
                .append("</"              )
                .append(TAG_PP_IMPORTANCE )
                .append(">"               );
            }

            if (mileage != null) {
                xmlMsg.append("<"            )
                .append(TAG_PP_MILEAGE )
                .append(">"            )
                .append(mileage        )
                .append("</"           )
                .append(TAG_PP_MILEAGE )
                .append(">"            );
            }

            if (owner != null) {
                xmlMsg.append("<"          )
                .append(TAG_PP_OWNER )
                .append(">"          )
                .append(owner        )
                .append("</"         )
                .append(TAG_PP_OWNER )
                .append(">"          );
            }

            if (percentComplete != null) {
                xmlMsg.append("<"                     )
                .append(TAG_PP_PERCENT_COMPLETE )
                .append(">"                     )
                .append(percentComplete         )
                .append("</"                    )
                .append(TAG_PP_PERCENT_COMPLETE )
                .append(">"                     );
            }

            xmlMsg.append("<"                            )
            .append(TAG_PP_REMINDER_SET            )
            .append(">"                            )
            .append(booleanToSIFValue(reminderSet) )
            .append("</"                           )
            .append(TAG_PP_REMINDER_SET            )
            .append(">"                            );

            if (reminderTime != null) {
                xmlMsg.append("<"                                  )
                .append(TAG_PP_REMINDER_TIME                 )
                .append(">"                                  )
                .append(formatDate(reminderTime, PATTERN_ISO8601_FORMAT, timeZone))
                .append("</"                                 )
                .append(TAG_PP_REMINDER_TIME                 )
                .append(">"                                  );
            } else {
                xmlMsg.append("<"                  )
                .append(TAG_PP_REMINDER_TIME )
                .append(">"                  )
                .append("</"                 )
                .append(TAG_PP_REMINDER_TIME )
                .append(">"                  );
            }

            if (sensitivity != null) {
                xmlMsg.append("<"                )
                .append(TAG_PP_SENSITIVITY )
                .append(">"                )
                .append(sensitivity        )
                .append("</"               )
                .append(TAG_PP_SENSITIVITY )
                .append(">"                );
            }

            if (status != null) {
                xmlMsg.append("<"           )
                .append(TAG_PP_STATUS )
                .append(">"           )
                .append(status        )
                .append("</"          )
                .append(TAG_PP_STATUS )
                .append(">"           );
            }

            if (subject != null) {
                xmlMsg.append("<"            )
                .append(TAG_PP_SUBJECT )
                .append(">"            )
                .append(subject        )
                .append("</"           )
                .append(TAG_PP_SUBJECT )
                .append(">"            );
            }

            xmlMsg.append("<"                              )
            .append(TAG_PP_TEAM_TASK                 )
            .append(">"                              )
            .append(booleanToSIFValue(this.teamTask) )
            .append("</"                             )
            .append(TAG_PP_TEAM_TASK                 )
            .append(">"                              );

            if (textDescription != null) {
                xmlMsg.append("<")
                .append(TAG_PP_NOTE     )
                .append(">"             )
                .append(textDescription )
                .append("</"            )
                .append(TAG_PP_NOTE     )
                .append(">"             );
            }

            xmlMsg.append("<"                       )
            .append(TAG_PP_TOTAL_WORK         )
            .append(">"                       )
            .append(String.valueOf(totalWork) )
            .append("</"                      )
            .append(TAG_PP_TOTAL_WORK         )
            .append(">"                       );

            xmlMsg.append("</task>");

        } catch (Exception e) {
            throw new TaskParseException("Error composing task Device: " +
                    e.getMessage());
        }

        return xmlMsg.toString();
    }

    private Date getDate(String sDate, String formatPattern, TimeZone tz) throws Exception {
        if (sDate == null || sDate.equals("")) {
            return null;
        }
        SimpleDateFormat formatter;

        if (formatPattern==null) {
            formatPattern = PATTERN_ISO8601_FORMAT;
        }

        if (tz==null) {
            tz = TimeZone.getTimeZone("GMT");
        }

        formatter = new SimpleDateFormat(formatPattern);
        formatter.setTimeZone(tz);
        return formatter.parse(sDate);
    }

    private Date getDate(String sDate) throws Exception {
        return getDate(sDate, null, null);
    }

    /**
     * Returns the SIF representation of a boolean value.
     *
     * @param value boolean
     * @return "0" if the given boolean is false, "1" otherwise
     */
    private String booleanToSIFValue(boolean value) {
        if (value) {
            return "1";
        } else {
            return "0";
        }
    }

    /**
     * If the given string is "1" returns true, otherwise returns false
     * @param value String
     * @return boolean
     */
    private boolean sifValueToBoolean(String value) {
        return value != null && "1".equals(value);
    }

    public void copyCommonProperties(ItemTask otherTask) {

        if(otherTask != null) {
            this.setActualWork(otherTask.getActualWork());

            if(otherTask.getBillingInformation() != null)
                this.setBillingInformation(otherTask.getBillingInformation());

            if(otherTask.getCategories() != null)
                this.setCategories(otherTask.getCategories());

            if(otherTask.getCompanies() != null)
                this.setCompanies(otherTask.getCompanies());

            if(otherTask.getDate() != null)
                this.setDate(otherTask.getDate());

            this.setComplete(otherTask.isComplete());

            if(otherTask.getDateCompleted() != null)
                this.setDateCompleted(otherTask.getDateCompleted());

            if(otherTask.getDueDate() != null)
                this.setDueDate(otherTask.getDueDate());

            if(otherTask.getHref() != null)
                this.setHref(otherTask.getHref());

            if(otherTask.getId() != null)
                this.setId(otherTask.getId());

            if(otherTask.getImportance() != null)
                this.setImportance(otherTask.getImportance());

            if(otherTask.getLastModified() != null)
                this.setLastModified(otherTask.getLastModified());

            if(otherTask.getMileage() != null)
                this.setMileage(otherTask.getMileage());

            if(otherTask.getOwner() != null)
                this.setOwner(otherTask.getOwner());

            if(otherTask.getPercentComplete() != null)
                this.setPercentComplete(otherTask.getPercentComplete());

            this.setReminderSet(otherTask.getReminderSet());

            if(otherTask.getReminderTime() != null)
                this.setReminderTime(otherTask.getReminderTime());

            if(otherTask.getSensitivity() != null)
                this.setSensitivity(otherTask.getSensitivity());

            this.setState(otherTask.getState());

            if(otherTask.getStatus() != null)
                this.setStatus(otherTask.getStatus());

            if(otherTask.getSubject() != null)
                this.setSubject(otherTask.getSubject());

            this.setTeamTask(otherTask.getTeamTask());

            if(otherTask.getTextDescription() != null)
                this.setTextDescription(otherTask.getTextDescription());

            this.setTotalWork(otherTask.getTotalWork());

            if(otherTask.getUsername() != null)
                this.setUsername(otherTask.getUsername());

        }

    }

    /**
     * Merge this task with the given task
     * @param other ItemTask
     * @return true if the original task is changed
     */
    public MergeResult merge(ItemTask other) {

        MergeResult eventMergeResult = new MergeResult("Task");

        if (other == null) {
            throw new IllegalStateException("The given task must be not null");
        }

        //
        // MergeResult used for each fields
        //
        MergeResult result = null;

        // actualWork
        result = PDIMergeUtils.compareIntegers(new Integer(this.actualWork), new Integer(other.actualWork));
        if (result.isSetARequired()) {
            this.actualWork = other.actualWork;
        } else if (result.isSetBRequired()) {
            other.actualWork = this.actualWork;
        }
        eventMergeResult.addMergeResult(result, "ActualWork");

        //(...)

        return eventMergeResult;
    }

    private static String formatDate(Date date, String formatPattern, TimeZone tz) {
        SimpleDateFormat formatter;

        if (date==null) {
            return null;
        }

         if (formatPattern==null) {
            formatPattern = PATTERN_ISO8601_FORMAT;
        }

        if (tz==null) {
            tz = TimeZone.getTimeZone("GMT");
        }

        formatter = new SimpleDateFormat(formatPattern);
        formatter.setTimeZone(tz);
        return formatter.format(date);
    }

}
