/**
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.domino.security ;


import com.funambol.domino.admin.DominoConnectorConfig;
import java.util.logging.Logger;
import java.util.logging.Level;
import java.security.Principal;

import com.funambol.framework.security.Officer;
import com.funambol.framework.security.Sync4jPrincipal;
import com.funambol.framework.server.store.PersistentStore;
import com.funambol.framework.server.store.PersistentStoreException;
import com.funambol.framework.server.store.NotFoundException;
import com.funambol.server.config.Configuration;

import com.funambol.domino.util.DominoWrapper;
import com.funambol.domino.util.Def;
import com.funambol.domino.exception.DominoAccessException;
import com.funambol.domino.exception.DominoConfigException;
import com.funambol.framework.core.Authentication;
import com.funambol.framework.core.Cred;
import com.funambol.framework.server.Sync4jUser;
import com.funambol.framework.tools.Base64;
import com.funambol.framework.logging.Sync4jLogger;


import com.funambol.server.admin.UserManager;


/**
 * This is an implementation of the <i>Officier</i> interface.
 * It always authenticates and authorizes users and resource accesses.
 *
 * @author  Gilberto Migliavacca
 * @version $Id: DominoOfficer.java,v 1.1 2006/08/08 08:33:50 gibi Exp $
 */
public class DominoOfficer implements Officer, java.io.Serializable {

    // --------------------------------------------------------------- Constants
    private static final String DEFAULT_USER_ROLE = "sync_user";

    // ---------------------------------------------------------- Protected data

    protected Logger  log = Sync4jLogger.getLogger(Def.LOGGER_NAME);

    protected PersistentStore ps = null;

    protected UserManager      userManager     = null;

    // ------------------------------------------------------------- Public data

    // -------------------------------------------------------------- Properties

    /**
     * Has the last login failed for incorrect login/password?
     */
    private boolean loginFailed = false;

    public boolean isLoginFailed() {
        return loginFailed;
    }

    /**
     * Has the last login failed for expired temporary login?
     */
    private boolean loginExpired = false;

    /**
     * Which type of authetication does impose the server?
     */
    private String clientAuth = Cred.AUTH_TYPE_BASIC;

    public String getClientAuth() {
        return this.clientAuth;
    }
    public void setClientAuth(String clientAuth) {
        this.clientAuth = clientAuth;
    }

    /**
     * Which type of authetication use the server to send his credential?
     */
    private String serverAuth = Cred.AUTH_NONE;

    public String getServerAuth() {
        return this.serverAuth;
    }

    public void setServerAuth(String serverAuth) {
        this.serverAuth = serverAuth;
    }


    // ---------------------------------------------------------- Public methods

    /**
     * Authenticates a credential.
     *
     * @param credential the credential to be authenticated
     *
     * @return true if the credential is autenticated, false otherwise
     */
    public boolean authenticate(Cred credential) {
        Configuration config = Configuration.getConfiguration();
        ps = config.getStore();

        userManager     = (UserManager) config.getUserManager();

        String type = credential.getType();

        if ((Cred.AUTH_TYPE_BASIC).equals(type)) {
            return authenticateBasicCredential(credential);
        }
        return false;
    }

    private boolean authenticateBasicCredential(Cred credential) {

        String username = null, password = null;
        Sync4jPrincipal principal = null;

        Authentication auth = credential.getAuthentication();
        String deviceId     = auth.getDeviceId();
        String userpwd      = new String(Base64.decode(auth.getData()));

        int p = userpwd.indexOf(':');

        if (p == -1) {
            username = userpwd;
            password = "";
        } else {
            username = (p>0) ? userpwd.substring(0, p) : "";
            password = (p == (userpwd.length()-1)) ? "" : userpwd.substring(p+1);
        }

        if (log.isLoggable(Level.FINEST)) {
            log.finest("Username: " + username);
        }

        //
        // Verify that exist the principal for the
        // specify deviceId and username
        //
        boolean principalFound = false;
        try {
            principal = Sync4jPrincipal.createPrincipal(username, deviceId);
            ps.read(principal);
            credential.getAuthentication().setPrincipalId(principal.getId());
            principalFound = true;
        } catch (NotFoundException nfe) {
            if (log.isLoggable(Level.FINEST)) {
                log.finest("Principal for " + username + ":" + deviceId + " not found");
            }
        } catch(PersistentStoreException e) {
            log.severe("Error reading principal: " + e);
            log.throwing(getClass().getName(), "authenticateBasicCredential", e);
            return false;
        }

        //
        // Verify these credentials on Domino
        //
        boolean isDominoUser = checkDominoCredentials(username, password);

        if (isDominoUser) {

            try {
                insertUser(username, " ");
                if (log.isLoggable(Level.FINEST)) {
                    log.info("User '" + username + "' created");
                }
            } catch (PersistentStoreException e) {
                log.severe("Error inserting a new user: " + e);
                log.throwing(getClass().getName(),
                             "authenticateBasicCredential", e);
                return false;
            }
            
            // We must create a new principal
            if (!principalFound) {
                principal = Sync4jPrincipal.createPrincipal(username, deviceId);
                try {
                    ps.store(principal);
                } catch (PersistentStoreException ex) {
                    log.severe("Error creating new principal: " + ex);
                    log.throwing(getClass().getName(), "authenticateBasicCredential", ex);
                    return false;
                }
                credential.getAuthentication().setPrincipalId(principal.getId());
            }

        }

        return isDominoUser;
    }


    /**
     * Authorizes a resource.
     *
     * @param principal the requesting entity
     * @param resource the name (or the identifier) of the resource to be authorized
     *
     * @return true if the credential is authorized to access the resource, false
     *         otherwise
     */
    public boolean authorize(Principal principal, String resource) {
        return true;
    }

    /**
     * Un-authenticates a credential.
     * Do nothing. In the current implementation, the authentication is
     * discarde as soon as the LoginContext is garbage collected.
     *
     * @param credential the credential to be unauthenticated
     */
    public void unAuthenticate(Cred credential) {
    }

    /**
     * @see com.funambol.framework.security.Officer
     *
     * @return boolean
     */
    public boolean isAccountExpired() {
        return loginExpired;
    }

    // --------------------------------------------------------- Private Methods

    /**
     * Insert a new user with the given username and password
     * @param userName the username
     * @param password the password
     * @throws PersistentStoreException if an error occurs
     */
    private void insertUser(String userName, String password) throws
            PersistentStoreException {

        Sync4jUser user = new Sync4jUser();
        user.setUsername(userName);
        user.setPassword(password);
        user.setRoles(new String[] {DEFAULT_USER_ROLE});
        userManager.insertUser(user);

    }

    /**
     * Checks if the given login and password are valid
     * credentials on domino server
     *
     *
     * @param server String
     * @param port int
     * @param login String
     * @param password String
     * @return boolean
     */
    private boolean checkDominoCredentials(String login, String password) {


        String  server = "localhost";
        int     port   = 63148;
        boolean ssl    = false;

        try {

          server = DominoConnectorConfig.getConfigInstance().getHost();
          port   = DominoConnectorConfig.getConfigInstance().getPort();
          ssl    = DominoConnectorConfig.getConfigInstance().getSsl();

        } catch (DominoConfigException e) {
            log.severe("Error Getting Connector Parameters:: " + e);
            log.throwing(getClass().getName(), "checkDominoCredentials", e);
        }

        DominoWrapper dw = new DominoWrapper();
        boolean isconnected = false;
        boolean error       = false;
        try {
            isconnected = dw.setSession(server, port, login, password, ssl);
        } catch (DominoAccessException dae){
            log.severe("Error getting connection: " + dae);
            log.throwing(getClass().getName(), "checkDominoCredentials", dae);
            error = true;
        }
        try {
            if (!error){
                dw.releaseSession();
            }
        }catch (DominoAccessException dae){
            log.severe("Error releaseing connection: " + dae);
            log.throwing(getClass().getName(), "checkDominoCredentials", dae);
        }
        return isconnected;
    }

}
